#include <stdio.h>
#include <string.h>
#include <unistd.h>
#include "kt10v2int.h"

#define TEST_CMD(_fnc_)\
    if ((res = (_fnc_))) {\
        printf("Command %s error(code=%d)!\n",__FUNCTION__,res);\
        return -1;\
    }

static int disp_fw_version(int id_conn)
{
    char version[32];
    int res;

    TEST_CMD(kt10v2_get_fw_version(id_conn, version));

    printf("FW version:%s\n",version);
}

static int disp_serial_no(int id_conn)
{    
    char serial[32];
    int res;

    TEST_CMD(kt10v2_get_serial(id_conn, serial));

    printf("Serial No:%s\n",serial);
}

static int disp_time(int id_conn)
{
    kt10v2_time_t time;
    int res;

    TEST_CMD(kt10v2_get_time(id_conn, &time));

    printf("Date:%d/%d/%d\n",time.day,time.month,time.year);
    printf("Time:%d:%.2d:%.2d\n",time.hour,time.minute,time.second);
}

static int download_record(int id_conn, int index, kt10v2_meas_log_entry_t *entry)
{
    int res;
    int length = sizeof(kt10v2_meas_log_entry_t);
    char fn[32];

    int pos,act_len;
    uint8_t *ptr;

    if (index == KT10V2_LAST_ENTRY) {
        snprintf(fn,sizeof(fn),"meas/last.dat");
    }
    else
    {
        snprintf(fn,sizeof(fn),"meas/meas%.4d.dat",index);
    }

    pos = 0;
    ptr = (uint8_t*)entry;

    while (pos < length) {
        TEST_CMD(kt10v2_read_file(id_conn, fn, ptr, pos, 450,  &act_len, 1));

        if (!act_len) return -1;

        pos += act_len;
        ptr += act_len;
    }

    return 0;
}

int download_all_meas(int id_conn)
{
    kt10v2_entry_table_rec_t  entry_table[65536];
    kt10v2_meas_log_entry_t   entry;
    kt10v2_params_t           params;
    int                       num_entries;
    int                       res, i, j, k;

    memset(entry_table,0,sizeof(entry_table));

    TEST_CMD(kt10v2_get_params(id_conn, &params, 1));
    printf("Downloading entry table(%d entries)...\n", params.max_stored_meas);

    //Download entry table - table containing information about index usage
    //if entry is non zero, index is occupied by download_record
    //higher number of index means newer record entry

    j = 0;
    while (j < params.max_stored_meas) {
        TEST_CMD(kt10v2_get_meas_log_entry_list(id_conn,j,&entry_table[j],1));

        j += KT10V2_ENTRY_TABLE_SIZE;
    }
    printf("Done.\n\n");

    //display entry table
    printf("Entries:\n");
    num_entries = 0;
    for (i=0;i<params.max_stored_meas;i++) {
        if ((int)entry_table[i].id > 0) {
            printf("    [%.4d] : %d ", i, entry_table[i].id);

            if (entry_table[i].flags & 0x01) printf(" note");

            printf("\n");

            num_entries++;
        }
    }
    printf("\n\n");

    //traverse through entry table, always find minimal index and then download it.
    //then zero index and repeat. cycle will donload all records in ascending order
    //display data

    while (1) {
        unsigned int min_id = 0xFFFFFFFF;
        int min_id_index = -1;

        for (j=0;j<params.max_stored_meas;j++) {
            if (entry_table[j].id) {
                if (entry_table[j].id < min_id) {
                    min_id = entry_table[j].id;
                    min_id_index = j;
                }
            }
        }

        if (min_id_index == -1) break;
        entry_table[min_id_index].id = 0;

        res = download_record(id_conn,min_id_index,&entry);
        if (res) {
            printf("\nError downloading data!\n");
            return -1;
        }

        char units_susc[20];
        char units_cond[20];

        strcpy(units_susc,"SI");
        strcpy(units_cond,"S/m");
        if (entry.type & 0x00FF00) strcpy(units_susc,"%"); else if (entry.type & 0x04000000) strcpy(units_susc,"CGS");
        if (entry.type & 0xFF0000) strcpy(units_cond,"%"); else if (entry.type & 0x08000000) strcpy(units_cond,"Ohm.m");

        if ((entry.type & 0x03) == 0x01) {
            printf("%d: %d %d %.4d/%.2d/%.2d %.2d:%.2d:%.2d Type=%d,CurveS=%d,CurveC=%d,Pin=%d,Error=%d,Dia=%2d    Susc=%6.3f%s Cond=%6.1f%s [%2d/%2d]\n",entry.id,entry.meas_mode,entry.avg_id,entry.year,entry.month,entry.day,entry.hour,entry.minute,entry.second,entry.type & 0x03, (entry.type >> 8) & 0xFF,(entry.type >> 16) & 0xFF,entry.pin,entry.error,entry.diameter,entry.susc,units_susc,entry.cond,units_cond,entry.meas_data.trigger_time,entry.meas_data.meas_time);
        }
        else
        if ((entry.type & 0x03) == 0x02) {
            printf("%d: %d %.4d/%.2d/%.2d %.2d:%.2d:%.2d Type=%d,CurveS=%d,CurveC=%d [%2d]\n",
                   entry.id,entry.meas_mode,entry.year,entry.month,entry.day,entry.hour,entry.minute,entry.second,entry.type & 0x03, (entry.type >> 8) & 0xFF,(entry.type >> 16) & 0xFF,entry.meas_data.meas_time);
            k = 0;

            while (entry.meas_data.meas_time) {
                printf("                ");

                for (j=0;j<5;j++) {
                    char c1='(';
                    char c2=')';
                    if (entry.meas_data.trigger_data[k/8] & (1 << (k%8))) {
                        c1='[';
                        c2=']';
                    }

                    printf("%c%8.3f %6.1f%c",c1,entry.meas_data.data.f[KT10V2_DATA_SUSC][k],entry.meas_data.data.f[KT10V2_DATA_COND][k],c2);
                    k++;

                    if (k == entry.meas_data.meas_time) break;
                }

                if (k == entry.meas_data.meas_time) break;

                printf("\n");
            }

            printf("\n");
        }
    }
}

int main()
{
    kt10v2_conn_string_t dev_list[32];
    int dev_found;
    
    //we have to init internal arrays and counters
    kt10v2_library_init();
   
    //this routine lists all KT10 devices connected over USB and puts their connection string into dev_list array
    //connection string is simple human readable text, which encapsulates connection type (usb, serial port,
    //bluetooth, tcp/ip) and device identifier (serial number for USB, IP address for TCP/IP etc).
    //it usually needs to be constructed manually, but USB (and BT) can be enumerated.
    //second argument limits length of the dev_list buffer to prevent overflow
    //last argument tells, how many devices were found

    kt10v2_list_usb_devices(dev_list, 32, &dev_found);

    int i;
    if (dev_found) {
        printf("Found devices:\n");

        //just print discovered the devices connection strings
        for (i=0;i<dev_found;i++) {
            printf("   %s\n",dev_list[i]);
        }    

        //and connect to first discovered for example
        //library can connect up to 255 devices simultaneously
        //each connection is identified by id_conn parameter, which is created here
        //if connection has been estabilished succesfuly
        //last parameter is timeout in ms used mainly for BT or remote connection
        //can be specified low for local USB

        int id_conn;
        if (kt10v2_connect(dev_list[0],  &id_conn, 500) == KT10V2_STATUS_OK) {
            printf("\n\nDevice connected.\n\n");

            //so the device is connected, let's do something
            //all following commands always start with id_conn, so multiple connections can be handled

            //let's read version
            disp_fw_version(id_conn);

            //let's read serial number
            disp_serial_no(id_conn);

            //let's read time
            disp_time(id_conn);

            printf("\n\n");

            //lets' download all measurements
            download_all_meas(id_conn);

            //let's disconnect
            kt10v2_disconnect(id_conn);

            printf("\n\nDevice disconnected.\n");
        }        
        else
        {
            printf("Device connection error!\n");
        }
    }
    else
    {
        printf("No device found!\n");
    }
    
    //and tidy up
    kt10v2_library_shut();
 
    return 0;
}
