#ifndef KT10V2_INT_H
#define KT10V2_INT_H

#include <stdint.h>

#ifdef WIN32
#ifdef _LIB
# define EXPORT __declspec (dllexport)
#else
# define EXPORT
#endif
#endif

#ifdef __linux__
#define EXPORT 
#endif


#ifdef __cplusplus
extern "C" {
#endif

#define KT10V2_MAX_CONNECTION_STR_LEN         128
    
#define KT10V2_STATUS_OK                      0x00
#define KT10V2_STATUS_ERROR                   0xFF

#define KT10V2_KAPPA_MAX_STORED_DATA          480

#define KT10V2_LAST_ENTRY                     -1
#define KT10V2_ENTRY_TABLE_SIZE               32

#define KT10V2_FILEOP_MKDIR                   0x01
#define KT10V2_FILEOP_CREATEFILE              0x02
#define KT10V2_FILEOP_APPENDFILE              0x03
#define KT10V2_FILEOP_DELETEFILE              0x04

#define KT10V2_STATE_MAIN_MENU                0x02
#define KT10V2_STATE_MAIN_MENU_INIT           0x19
#define KT10V2_STATE_MEAS_PROG                0x1A
#define KT10V2_STATE_SCAN_PROG                0x1B

#define KT10V2_KEY_A_SHORT                    0x2001
#define KT10V2_KEY_B_SHORT                    0x2002
#define KT10V2_KEY_A_LONG                     0x4001
#define KT10V2_KEY_B_LONG          	      0x4002

#define KT10V2_DATA_SUSC                      0
#define KT10V2_DATA_COND                      1

#define KT10V2_DATA_FREQ                      0
#define KT10V2_DATA_AMPL                      1
    

typedef char kt10v2_conn_string_t[KT10V2_MAX_CONNECTION_STR_LEN];

typedef struct gt40_time {
    uint32_t               year;
    uint32_t               month;
    uint32_t               day;
    uint32_t               hour;
    uint32_t               minute;
    uint32_t               second;
} kt10v2_time_t;

typedef struct kt10v2_gps_coords {
    uint32_t               flags;
    uint32_t               latitude;
    uint32_t               longitude;
    uint32_t               altitude;
} kt10v2_gps_coords_t;

typedef struct kt10v2_entry_table_rec {
    uint32_t                id;
    uint8_t                 flags;
} __attribute__((packed)) kt10v2_entry_table_rec_t;

typedef struct kt10v2_status {
    uint32_t                error;
    uint32_t                bat_voltage;
    uint16_t                bat_perc;
    uint16_t                contrast;
    uint16_t                bt_status;
    uint16_t                gps_status;
    uint32_t                flash_size;
} __attribute__((packed)) kt10v2_status_t;

typedef struct kt10v2_calib {
    float                   nom_susc[2];
    float                   nom_cond[2];
    float                   fs,fc;
    float                   us,uc;
} __attribute__((packed)) kt10v2_calib1_t;

typedef struct kt10v2_caliv2 {
    float                   a,b,c,d;
    float                   exp_a,exp_b,exp_c,exp_d;	
    float                   cond_nom[8];
    float                   cond_meas[8];	
} __attribute__((packed)) kt10v2_calib2_t;

typedef struct kt10v2_params {
    uint8_t                 use_lp;
    uint8_t                 audio_vol;
    uint8_t                 demag_corr;
    uint8_t                 calib_mode;
    kt10v2_calib1_t         calib1;
    kt10v2_calib1_t         calib1_pin;
    uint8_t                 pc_auth;
    uint8_t                 gps_pin_enable;
    uint8_t                 gps_on;
    int8_t                  gps_pin[8];
    uint8_t                 gps_paired;
    uint16_t                gps_uuid;
    uint8_t                 gps_addr[6];
    uint8_t                 reserved;
    uint16_t                sd_timeout;
    uint8_t                 replay_note;
    uint8_t                 save_error_meas;
    uint8_t                 battery_type;
    uint32_t                max_stored_meas;
    uint8_t                 use_conc_lut_cond;
    uint8_t                 save_to_int;
    uint8_t                 use_conc_lut_susc;
    uint8_t                 diameter;
    uint8_t                 contrast;
    uint8_t                 meas_mode;
    uint8_t                 pin;
    uint32_t                bt_baud_rate;
    uint32_t                bh_units;
    uint32_t                bh_mode;
    uint32_t                cond_corr;
    uint32_t                adv_mode;
    uint8_t                 susc_units;
    uint8_t                 cond_units;
    kt10v2_calib2_t         calib2;
    kt10v2_calib2_t         calib2_pin;	
    uint32_t                filler[1000];
} __attribute__((packed)) kt10v2_params_t;

typedef struct kt10v2_meas_data {
    uint32_t                ud;
    uint32_t                trigger_time;
    uint32_t                meas_time;
    uint8_t                 trigger_data[KT10V2_KAPPA_MAX_STORED_DATA/8];
    union {
        float               f[2][KT10V2_KAPPA_MAX_STORED_DATA];
        uint32_t            i[2][KT10V2_KAPPA_MAX_STORED_DATA];
    } __attribute__((packed)) data;
} __attribute__((packed)) kt10v2_meas_data_t;

typedef struct kt10v2_bore_hole_rec {
    uint32_t                id;
    int8_t                  desc[16];
    uint8_t                 units;
    float                   start_depth;
    float                   end_depth;
    float                   core_length;
    float                   depth_interval;
    uint32_t                cores_per_box;
    uint32_t                num_cores;
    uint32_t                num_boxes;
    int32_t                 cur_core;
    uint32_t                header_index;
    uint32_t                flags;
    float                   cur_depth;
} kt10v2_bore_hole_rec_t;

typedef struct kt10v2_meas_log_entry {
    uint32_t                magic;
    uint32_t                id;
    uint32_t                avg_id;
    uint16_t                year;
    uint8_t                 month;
    uint8_t                 day;
    uint8_t                 hour;
    uint8_t                 minute;
    uint8_t                 second;
    uint32_t                type;
    uint8_t                 pin;
    uint8_t                 error;
    uint8_t                 diameter;
    kt10v2_gps_coords_t     gps_coords;
    uint8_t  	            meas_mode;
    float                   susc;
    float                   cond;
    kt10v2_calib1_t         calib;
    kt10v2_meas_data_t      meas_data;
    kt10v2_bore_hole_rec_t  bore_hole_rec;
} __attribute__((packed)) kt10v2_meas_log_entry_t;


extern EXPORT int kt10v2_library_init(void);
extern EXPORT int kt10v2_library_shut(void);
extern EXPORT int kt10v2_list_usb_devices(kt10v2_conn_string_t *list, int max_devices, int *dev_count);
extern EXPORT int kt10v2_connect(kt10v2_conn_string_t conn_string, int *id_conn, int timeout);
extern EXPORT int kt10v2_disconnect(int id_conn);
extern EXPORT int kt10v2_get_serial(int id_conn, char *serial);
extern EXPORT int kt10v2_get_fw_version(int id_conn, char *version);
extern EXPORT int kt10v2_get_time(int id_conn, kt10v2_time_t *time);
extern EXPORT int kt10v2_set_time(int id_conn, kt10v2_time_t *time);
extern EXPORT int kt10v2_restart(int id_conn);

extern EXPORT int kt10v2_get_params(int id, kt10v2_params_t *params, int wait_rsp);
extern EXPORT int kt10v2_set_params(int id, const kt10v2_params_t *params, int wait_rsp);
extern EXPORT int kt10v2_get_status(int id, kt10v2_status_t *status, int wait_rsp);

extern EXPORT int kt10v2_get_meas_log_entry_list(int id, int32_t first, kt10v2_entry_table_rec_t *entry_table, int wait_rsp);

extern EXPORT int kt10v2_file_info(int id, const char *file_name, int32_t *length, int wait_rsp);
extern EXPORT int kt10v2_write_file(int id, const char *file_name, int32_t flags, uint8_t *buff, int32_t length, int wait_rsp);
extern EXPORT int kt10v2_read_file(int id, const char *file_name, uint8_t *buff, int32_t pos, int32_t length, int32_t *act_len, int wait_rsp);

extern EXPORT int kt10v2_clear_logs(int id, int32_t region, int wait_rsp);

extern EXPORT int kt10v2_send_event(int id, int32_t event, int wait_rsp);
extern EXPORT int kt10v2_set_state(int id, int32_t state, int wait_rsp);
extern EXPORT int kt10v2_get_state(int id, int32_t *state, int wait_rsp);

extern EXPORT int kt10v2_get_screen(int id, uint8_t *screen, int wait_rsp);

#ifdef __cplusplus
}
#endif

#endif
